@@ -24,10 +24,8 @@
 typedef enum
 {
   GEGL_RUN_MODE_HELP,
-  GEGL_RUN_MODE_DOT,
   GEGL_RUN_MODE_EDITOR,
-  GEGL_RUN_MODE_PNG,
-  GEGL_RUN_MODE_PPM,
+  GEGL_RUN_MODE_OUTPUT,
   GEGL_RUN_MODE_XML
 } GeglRunMode;
 
