@@ -16,7 +16,6 @@
  */
 
 #include <string.h>
-#include "config.h"
 #include <glib.h>
 #include <gegl.h>
 #include <gtk/gtk.h>
@@ -24,7 +23,7 @@
 #include "util/gegl-view.c"
 #include "property-types/gegl-path.h"
 
-#define HARDNESS     0.2
+#define HARDNESS     0.6
 #define LINEWIDTH   60.0
 #define COLOR       "rgba(0.0,0.0,0.0,0.4)"
 
@@ -51,10 +50,10 @@ static gboolean paint_press (GtkWidget      *widget,
       over       = gegl_node_new_child (gegl, "operation", "gegl:over", NULL);
       stroke     = gegl_node_new_child (gegl, "operation", "gegl:path",
                                         "d", vector,
+                                        "fill-opacity", 0.0,
                                         "stroke", gegl_color_new (COLOR),
-                                        "fill", gegl_color_new ("none"),
                                         "stroke-width", LINEWIDTH,
-                                        "hardness", HARDNESS,
+                                        "stroke-hardness", HARDNESS,
                                         NULL);
       gegl_node_link_many (top, over, out, NULL);
       gegl_node_connect_to (stroke, "output", over, "aux");
@@ -111,7 +110,7 @@ static gboolean paint_release (GtkWidget      *widget,
       processor = gegl_node_new_processor (writebuf, &roi);
       while (gegl_processor_work (processor, NULL)) ;
 
-      g_object_unref (processor);
+      gegl_processor_destroy (processor);
       g_object_unref (writebuf);
 
       gegl_node_link_many (top, out, NULL);
@@ -131,7 +130,7 @@ gint
 main (gint    argc,
       gchar **argv)
 {
-
+  g_thread_init (NULL);
   gtk_init (&argc, &argv);
   gegl_init (&argc, &argv);
 
@@ -143,9 +142,15 @@ main (gint    argc,
       GeglRectangle rect = {0, 0, 512, 512};
       gpointer buf;
 
-      buffer = gegl_buffer_new (&rect, babl_format("RaGaBaA float"));
+      /* XXX: for best overall performance, this format should probably
+       * be RaGaBaA float, overeager in-place processing code makes that fail though.
+       */
+      buffer = gegl_buffer_new (&rect, babl_format("R'G'B' u8"));
+      /* it would be useful to have a programmatic way of doing this, filling
+       * with a given pixel value
+       */
       buf    = gegl_buffer_linear_open (buffer, NULL, NULL, babl_format ("Y' u8"));
-      memset (buf, 255, 512 * 512); /* FIXME: we need a babl_buffer_paint () */
+      memset (buf, 255, 512 * 512);
       gegl_buffer_linear_close (buffer, buf);
     }
   else
@@ -164,16 +169,14 @@ main (gint    argc,
     top  = loadbuf;
   }
 
-  gtk_signal_connect (GTK_OBJECT (view), "motion-notify-event",
-                      (GtkSignalFunc) paint_motion, NULL);
-  gtk_signal_connect (GTK_OBJECT (view), "button-press-event",
-                      (GtkSignalFunc) paint_press, NULL);
-  gtk_signal_connect (GTK_OBJECT (view), "button-release-event",
-                      (GtkSignalFunc) paint_release, NULL);
-
+  g_signal_connect (GTK_OBJECT (view), "motion-notify-event",
+                    (GCallback) paint_motion, NULL);
+  g_signal_connect (GTK_OBJECT (view), "button-press-event",
+                    (GCallback) paint_press, NULL);
+  g_signal_connect (GTK_OBJECT (view), "button-release-event",
+                    (GCallback) paint_release, NULL);
   gtk_widget_add_events (view, GDK_BUTTON_RELEASE_MASK);
 
-
   gtk_container_add (GTK_CONTAINER (window), view);
   gtk_widget_set_size_request (view, 512, 512);
 
@@ -181,7 +184,6 @@ main (gint    argc,
                     G_CALLBACK (gtk_main_quit), window);
   gtk_widget_show_all (window);
 
-
   gtk_main ();
   g_object_unref (gegl);
   gegl_buffer_destroy (buffer);
