@@ -82,10 +82,8 @@ gegl_buffer_linear_new_from_data (const gpointer       data,
     buffer = gegl_buffer_linear_new2 (extent, format, rowstride);
 
   {
-    GeglTile *tile = g_object_new (GEGL_TYPE_TILE, NULL);
+    GeglTile *tile = gegl_tile_new_bare (); 
 
-    tile->rev        = 1;
-    tile->stored_rev = 1;
     tile->tile_storage = buffer->tile_storage;
     tile->x = 0;
     tile->y = 0;
@@ -95,12 +93,9 @@ gegl_buffer_linear_new_from_data (const gpointer       data,
     tile->next_shared = tile;
     tile->prev_shared = tile;
 
-    {
-      GeglTileHandlerCache *cache = g_object_get_data (G_OBJECT (buffer->tile_storage), "cache");
-      if (cache)
-        gegl_tile_handler_cache_insert (cache, tile, 0, 0, 0);
-    }
-    g_object_unref (tile);
+    if (buffer->tile_storage->cache)
+      gegl_tile_handler_cache_insert (buffer->tile_storage->cache, tile, 0, 0, 0);
+    gegl_tile_unref (tile);
   }
 
   return buffer;
@@ -132,6 +127,8 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
   if (extent == NULL)
     extent=&buffer->extent;
 
+  /*gegl_buffer_lock (buffer);*/
+  g_mutex_lock (buffer->tile_storage->mutex);
   if (extent->x     == buffer->extent.x &&
       extent->y     == buffer->extent.y &&
       extent->width == buffer->tile_width &&
@@ -151,7 +148,6 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
       tile = gegl_tile_source_get_tile ((GeglTileSource*) (buffer),
                                         0,0,0);
       g_assert (tile);
-      gegl_buffer_lock (buffer);
       gegl_tile_lock (tile);
 
       g_object_set_data (G_OBJECT (buffer), "linear-tile", tile);
@@ -179,6 +175,7 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
               )
             {
               info->refs++;
+              g_print ("!!!!!! sharing a linear buffer!!!!!\n");
               return info->buf;
             }
         }
@@ -192,14 +189,14 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
     linear_buffers = g_list_append (linear_buffers, info);
     g_object_set_data (G_OBJECT (buffer), "linear-buffers", linear_buffers);
 
-    info->extent = buffer->extent;
+    info->extent = *extent;
     info->format = format;
 
     rs = info->extent.width * babl_format_get_bytes_per_pixel (format);
     if(rowstride)*rowstride = rs;
 
     info->buf = gegl_malloc (rs * info->extent.height);
-    gegl_buffer_get (buffer, 1.0, &info->extent, format, info->buf, rs);
+    gegl_buffer_get_unlocked (buffer, 1.0, &info->extent, format, info->buf, rs);
     return info->buf;
   }
   return NULL;
@@ -214,7 +211,6 @@ gegl_buffer_linear_close (GeglBuffer *buffer,
   if (tile)
     {
       gegl_tile_unlock (tile);
-      gegl_buffer_unlock (buffer);
       g_object_set_data (G_OBJECT (buffer), "linear-tile", NULL);
       tile = NULL;
     }
@@ -233,11 +229,17 @@ gegl_buffer_linear_close (GeglBuffer *buffer,
               info->refs--;
 
               if (info->refs>0)
+                {
+                  g_print ("EEeeek! %s\n", G_STRLOC);
                 return; /* there are still others holding a reference to
                          * this linear buffer
                          */
+                }
 
+              g_mutex_unlock (buffer->tile_storage->mutex);
+              /* XXX: potential race */
               gegl_buffer_set (buffer, &info->extent, info->format, info->buf, 0);
+              g_mutex_lock (buffer->tile_storage->mutex);
               break;
             }
           else
@@ -255,5 +257,7 @@ gegl_buffer_linear_close (GeglBuffer *buffer,
 
       g_object_set_data (G_OBJECT (buffer), "linear-buffers", linear_buffers);
     }
+  /*gegl_buffer_unlock (buffer);*/
+  g_mutex_unlock (buffer->tile_storage->mutex);
   return;
 }
