@@ -20,6 +20,7 @@
 #ifndef __GEGL_BUFFER_PRIVATE_H__
 #define __GEGL_BUFFER_PRIVATE_H__
 
+#include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
 #include "gegl-sampler.h"
 #include "gegl-tile-handler.h"
@@ -53,6 +54,7 @@ struct _GeglBuffer
   GeglSampler      *sampler; /* cached sampler for speeding up random
                                 access interpolated fetches from the
                                 buffer */
+  const Babl       *sampler_format; /* the format of the cached sampler */
 
   GeglTileStorage  *tile_storage;
 
@@ -70,6 +72,8 @@ struct _GeglBuffer
 
   gchar            *alloc_stack_trace; /* Stack trace for allocation,
                                           useful for debugging */
+
+  gpointer          backend;
 };
 
 struct _GeglBufferClass
@@ -78,37 +82,41 @@ struct _GeglBufferClass
 };
 
 
-const GeglRectangle* gegl_buffer_get_abyss  (GeglBuffer           *buffer);
 
-gint                 gegl_buffer_leaks       (void);
+gint              gegl_buffer_leaks       (void);
 
-void                 gegl_buffer_stats       (void);
+void              gegl_buffer_stats       (void);
 
-void                 gegl_buffer_save        (GeglBuffer          *buffer,
-                                              const gchar         *path,
-                                              const GeglRectangle *roi);
+void              gegl_buffer_save        (GeglBuffer          *buffer,
+                                           const gchar         *path,
+                                           const GeglRectangle *roi);
 
 
-const gchar         *gegl_swap_dir           (void);
+const gchar      *gegl_swap_dir           (void);
 
 
-void                 gegl_tile_cache_init    (void);
+void              gegl_tile_cache_init    (void);
 
-void                 gegl_tile_cache_destroy (void);
+void              gegl_tile_cache_destroy (void);
 
-GeglTileBackend    * gegl_buffer_backend     (GeglBuffer *buffer);
+GeglTileBackend * gegl_buffer_backend     (GeglBuffer *buffer);
 
-gboolean             gegl_buffer_is_shared   (GeglBuffer *buffer);
-
-gboolean             gegl_buffer_try_lock    (GeglBuffer *buffer);
-#if 0
-gboolean             gegl_buffer_lock        (GeglBuffer *buffer);
-gboolean             gegl_buffer_unlock      (GeglBuffer *buffer);
-#else
-#define gegl_buffer_lock(o)  {}
-#define gegl_buffer_unlock(o)  {}
-#endif
+gboolean          gegl_buffer_is_shared   (GeglBuffer *buffer);
 
+gboolean          gegl_buffer_try_lock    (GeglBuffer *buffer);
+gboolean          gegl_buffer_lock        (GeglBuffer *buffer);
+gboolean          gegl_buffer_unlock      (GeglBuffer *buffer);
+void              gegl_buffer_set_unlocked (GeglBuffer          *buffer,
+                                            const GeglRectangle *rect,
+                                            const Babl          *format,
+                                            void                *src,
+                                            gint                 rowstride);
+void              gegl_buffer_get_unlocked (GeglBuffer          *buffer,
+                                            gdouble              scale,
+                                            const GeglRectangle *rect,
+                                            const Babl          *format,
+                                            gpointer             dest_buf,
+                                            gint                 rowstride);
 
 GeglBuffer *
 gegl_buffer_new_ram (const GeglRectangle *extent,
@@ -125,4 +133,65 @@ void            gegl_buffer_sampler           (GeglBuffer     *buffer,
                                                gpointer        sampler);
 
 
+
+/* the instance size of a GeglTile is a bit large, and should if possible be
+ * trimmed down
+ */
+struct _GeglTile
+{
+ /* GObject          parent_instance;*/
+  gint             ref_count;
+
+  guchar          *data;        /* actual pixel data for tile, a linear buffer*/
+  gint             size;        /* The size of the linear buffer */
+
+  GeglTileStorage *tile_storage; /* the buffer from which this tile was
+                                  * retrieved needed for the tile to be able to
+                                  * store itself back (for instance when it is
+                                  * unreffed for the last time)
+                                  */
+  gint             x, y, z;
+
+
+  guint            rev;         /* this tile revision */
+  guint            stored_rev;  /* what revision was we when we from tile_storage?
+                                   (currently set to 1 when loaded from disk */
+
+  gchar            lock;        /* number of times the tile is write locked
+                                 * should in theory just have the values 0/1
+                                 */
+  GMutex          *mutex;
+
+  /* the shared list is a doubly linked circular list */
+  GeglTile        *next_shared;
+  GeglTile        *prev_shared;
+
+  void (*destroy_notify) (gpointer pixels,
+                          gpointer data);
+  gpointer         destroy_notify_data;
+};
+
+#ifndef __GEGL_TILE_C
+#define gegl_tile_get_data(tile)  ((guchar*)((tile)->data))
+#endif // __GEGL_TILE_C
+
+
+/* computes the positive integer remainder (also for negative dividends)
+ */
+#define GEGL_REMAINDER(dividend, divisor) \
+                   (((dividend) < 0) ? \
+                    (divisor) - 1 - ((-((dividend) + 1)) % (divisor)) : \
+                    (dividend) % (divisor))
+
+#define gegl_tile_offset(coordinate, stride) GEGL_REMAINDER((coordinate), (stride))
+
+/* helper function to compute tile indices and offsets for coordinates
+ * based on a tile stride (tile_width or tile_height)
+ */
+#define gegl_tile_indice(coordinate,stride) \
+  (((coordinate) >= 0)?\
+      (coordinate) / (stride):\
+      ((((coordinate) + 1) /(stride)) - 1))
+
+
 #endif
