@@ -37,6 +37,7 @@
 #include "gegl-buffer-types.h"
 #include "gegl-buffer.h"
 #include "gegl-buffer-private.h"
+#include "gegl-debug.h"
 #include "gegl-tile-storage.h"
 #include "gegl-tile-backend.h"
 #include "gegl-tile-handler.h"
@@ -64,7 +65,6 @@ typedef struct
   gint             tile_size;
   gint             offset;
   gint             entry_count;
-  GeglBufferBlock *last_added;
   GeglBufferBlock *in_holding; /* we need to write one block added behind
                                 * to be able to recompute the forward pointing
                                 * link from one entry to the next.
@@ -77,7 +77,7 @@ gegl_tile_entry_new (gint x,
                      gint y,
                      gint z)
 {
-  GeglBufferTile *entry = g_malloc (sizeof(GeglBufferTile));
+  GeglBufferTile *entry = g_malloc0 (sizeof(GeglBufferTile));
   entry->block.flags = GEGL_FLAG_TILE;
   entry->block.length = sizeof (GeglBufferTile);
 
@@ -194,16 +194,17 @@ gegl_buffer_header_init (GeglBufferHeader *header,
                          gint              tile_width,
                          gint              tile_height,
                          gint              bpp,
-                         Babl*             format)
+                         const Babl*       format)
 {
-  strcpy (header->magic, "GEGL");
+  memcpy (header->magic, "GEGL", 4);
 
   header->flags = GEGL_FLAG_HEADER;
   header->tile_width  = tile_width;
   header->tile_height = tile_height;
   header->bytes_per_pixel = bpp;
   {
-    gchar buf[64];
+    gchar buf[64] = { 0, };
+
     g_snprintf (buf, 64, "%s%c\n%i×%i %ibpp\n%ix%i\n\n\n\n\n\n\n\n\n", 
           babl_get_name (format), 0,
           header->tile_width,
@@ -224,9 +225,18 @@ gegl_buffer_save (GeglBuffer          *buffer,
 
   glong prediction = 0; 
   gint bpp;
+  gint tile_width;
+  gint tile_height;
 
   GEGL_BUFFER_SANITY;
 
+  if (! roi)
+    roi = &buffer->extent;
+
+  GEGL_NOTE (GEGL_DEBUG_BUFFER_SAVE,
+             "starting to save buffer %s, roi: %d,%d %dx%d",
+             path, roi->x, roi->y, roi->width, roi->height);
+
   /* a header should follow the same structure as a blockdef with
    * respect to the flags and next offsets, thus this is a valid
    * cast shortcut.
@@ -239,85 +249,70 @@ gegl_buffer_save (GeglBuffer          *buffer,
 #else
   info->o    = open (info->path, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
 #endif
+  tile_width  = buffer->tile_storage->tile_width;
+  tile_height = buffer->tile_storage->tile_height;
   g_object_get (buffer, "px-size", &bpp, NULL);
-  info->header.x           = buffer->extent.x;
-  info->header.y           = buffer->extent.y;
-  info->header.width       = buffer->extent.width;
-  info->header.height      = buffer->extent.height;
+
+  info->header.x           = roi->x;
+  info->header.y           = roi->y;
+  info->header.width       = roi->width;
+  info->header.height      = roi->height;
   gegl_buffer_header_init (&info->header,
-                           buffer->tile_storage->tile_width,
-                           buffer->tile_storage->tile_height,
+                           tile_width,
+                           tile_height,
                            bpp,
                            buffer->tile_storage->format
                            );
   info->header.next = (prediction += sizeof (GeglBufferHeader));
-
-
-  info->tile_size = info->header.tile_width  *
-                    info->header.tile_height *
-                    info->header.bytes_per_pixel;
+  info->tile_size = tile_width * tile_height * bpp;
 
   g_assert (info->tile_size % 16 == 0);
 
-  /* collect list of tiles to be written */
+  GEGL_NOTE (GEGL_DEBUG_BUFFER_SAVE,
+             "collecting list of tiles to be written");
   {
-    gint width       = buffer->extent.width;
-    gint height      = buffer->extent.height;
-    gint tile_width  = info->header.tile_width;
-    gint tile_height = info->header.tile_height;
-    gint x           = 0;
-    gint y           = 0;
+    gint z;
+    gint factor = 1;
+    int  bufy = roi->y;
 
-    int  bufy;
-
-    if (roi)
+    for (z = 0; z < 1; z++)
       {
-        x      = roi->x;
-        y      = roi->y;
-        width  = roi->width;
-        height = roi->height;
+        bufy = roi->y;
+        while (bufy < roi->y + roi->height)
+          {
+            gint tiledy  = roi->y + bufy;
+            gint offsety = gegl_tile_offset (tiledy, tile_height);
+            gint bufx    = roi->x;
+
+            while (bufx < roi->x + roi->width)
+              {
+                gint tiledx  = roi->x + bufx;
+                gint offsetx = gegl_tile_offset (tiledx, tile_width);
+
+                gint tx = gegl_tile_indice (tiledx / factor, tile_width);
+                gint ty = gegl_tile_indice (tiledy / factor, tile_height);
+
+                if (gegl_tile_source_exist (GEGL_TILE_SOURCE (buffer), tx, ty, z))
+                  {
+                    GeglBufferTile *entry;
+
+                    GEGL_NOTE (GEGL_DEBUG_BUFFER_SAVE,
+                               "Found tile to save, tx, ty, z = %d, %d, %d",
+                               tx, ty, z);
+
+                    entry = gegl_tile_entry_new (tx, ty, z);
+                    info->tiles = g_list_prepend (info->tiles, entry);
+                    info->entry_count++;
+                  }
+                bufx += (tile_width - offsetx) * factor;
+              }
+            bufy += (tile_height - offsety) * factor;
+          }
+        factor *= 2;
       }
-
-    {
-      gint z;
-      gint factor = 1;
-      for (z = 0; z < 1; z++)
-        {
-          bufy = y;
-          while (bufy < buffer->extent.y + height)
-            {
-              gint tiledy  = buffer->extent.y + bufy;
-              gint offsety = gegl_tile_offset (tiledy, tile_height);
-              gint bufx    = x;
-
-              while (bufx < buffer->extent.x + width)
-                {
-                  gint tiledx  = buffer->extent.x + bufx;
-                  gint offsetx = gegl_tile_offset (tiledx, tile_width);
-
-                  gint tx = gegl_tile_indice (tiledx / factor, tile_width);
-                  gint ty = gegl_tile_indice (tiledy / factor, tile_height);
-
-                  if (gegl_tile_source_exist (GEGL_TILE_SOURCE (buffer), tx, ty, z))
-                    {
-                      GeglBufferTile *entry;
-
-                      entry = gegl_tile_entry_new (tx, ty, z);
-                      info->tiles = g_list_prepend (info->tiles, entry);
-                      info->entry_count++;
-                    }
-                  bufx += (tile_width - offsetx) * factor;
-                }
-              bufy += (tile_height - offsety) * factor;
-            }
-          factor *= 2;
-        }
-    }
-
-
-  /* XXX:why was the list reversed here when it is just about to be sorted?
-   */
-/*    info->tiles = g_list_reverse (info->tiles);*/
+  GEGL_NOTE (GEGL_DEBUG_BUFFER_SAVE,
+             "size of list of tiles to be written: %d",
+             g_list_length (info->tiles));
   }
 
   /* sort the list of tiles into zorder */
@@ -395,7 +390,7 @@ gegl_buffer_save (GeglBuffer          *buffer,
             info->offset += ret;
         }
 #endif
-        g_object_unref (G_OBJECT (tile));
+        gegl_tile_unref (tile);
         i++;
       }
   }
