@@ -30,58 +30,9 @@
 #include "gegl-tile-handler-cache.h"
 #include "gegl-debug.h"
 
-static GQueue *cache_queue = NULL;
-static gint    cache_wash_percentage = 20;
-static gint    cache_hits = 0;
-static gint    cache_misses = 0;
-
-static gint    cache_total = 0;  /* approximate amount of bytes stored */
-
-struct _GeglTileHandlerCache
-{
-  GeglTileHandler parent_instance;
-  GSList *free_list;
-};
-
-void gegl_tile_cache_init (void)
-{
-  if (cache_queue == NULL)
-    cache_queue = g_queue_new ();
-}
-
-void gegl_tile_cache_destroy (void)
-{
-  if (cache_queue)
-    g_queue_free (cache_queue);
-  cache_queue = NULL;
-}
-
-static gboolean    gegl_tile_handler_cache_wash     (GeglTileHandlerCache *cache);
-
-static GeglTile *  gegl_tile_handler_cache_get_tile (GeglTileHandlerCache *cache,
-                                                     gint              x,
-                                                     gint              y,
-                                                     gint              z);
-
-static gboolean    gegl_tile_handler_cache_has_tile (GeglTileHandlerCache *cache,
-                                                     gint              x,
-                                                     gint              y,
-                                                     gint              z);
-void               gegl_tile_handler_cache_insert   (GeglTileHandlerCache *cache,
-                                                     GeglTile         *tile,
-                                                     gint              x,
-                                                     gint              y,
-                                                     gint              z);
-static void        gegl_tile_handler_cache_void     (GeglTileHandlerCache *cache,
-                                                     gint              x,
-                                                     gint              y,
-                                                     gint              z);
-static void        gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
-                                                     gint              x,
-                                                     gint              y,
-                                                     gint              z);
-
-G_DEFINE_TYPE (GeglTileHandlerCache, gegl_tile_handler_cache, GEGL_TYPE_TILE_HANDLER)
+/*
+#define GEGL_DEBUG_CACHE_HITS
+*/
 
 typedef struct CacheItem
 { 
@@ -93,40 +44,103 @@ typedef struct CacheItem
   gint      z;
 } CacheItem;
 
-static void
-finalize (GObject *object)
+struct _GeglTileHandlerCache
 {
-  GeglTileHandlerCache *cache;
-  cache = (GeglTileHandlerCache *) object;
+  GeglTileHandler parent_instance;
+  GSList *free_list;
+};
 
-  G_OBJECT_CLASS (gegl_tile_handler_cache_parent_class)->finalize (object);
+
+static void       gegl_tile_handler_cache_dispose    (GObject              *object);
+static gboolean   gegl_tile_handler_cache_wash       (GeglTileHandlerCache *cache);
+static gpointer   gegl_tile_handler_cache_command    (GeglTileSource       *tile_store,
+                                                      GeglTileCommand       command,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z,
+                                                      gpointer              data);
+static GeglTile * gegl_tile_handler_cache_get_tile   (GeglTileHandlerCache *cache,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z);
+static gboolean   gegl_tile_handler_cache_has_tile   (GeglTileHandlerCache *cache,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z);
+void              gegl_tile_handler_cache_insert     (GeglTileHandlerCache *cache,
+                                                      GeglTile             *tile,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z);
+static void       gegl_tile_handler_cache_void       (GeglTileHandlerCache *cache,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z);
+static void       gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
+                                                      gint                  x,
+                                                      gint                  y,
+                                                      gint                  z);
+
+
+static GStaticMutex mutex                 = G_STATIC_MUTEX_INIT;
+static GQueue      *cache_queue           = NULL;
+static GHashTable  *cache_ht              = NULL;
+static gint         cache_wash_percentage = 20;
+static gint         cache_total           = 0; /* approximate amount of bytes stored */
+#ifdef GEGL_DEBUG_CACHE_HITS
+static gint         cache_hits            = 0;
+static gint         cache_misses          = 0;
+#endif
+
+
+G_DEFINE_TYPE (GeglTileHandlerCache, gegl_tile_handler_cache, GEGL_TYPE_TILE_HANDLER)
+
+
+static void
+gegl_tile_handler_cache_class_init (GeglTileHandlerCacheClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+
+  gobject_class->dispose = gegl_tile_handler_cache_dispose;
+}
+
+static void
+gegl_tile_handler_cache_init (GeglTileHandlerCache *cache)
+{
+  ((GeglTileSource*)cache)->command = gegl_tile_handler_cache_command;
+  gegl_tile_cache_init ();
 }
 
 static void
 gegl_tile_handler_cache_dispose_buffer_tiles (gpointer itm,
                                               gpointer userdata)
 {
-  CacheItem *item = itm;
+  CacheItem *item;
+  item = itm;
   if (item->handler == userdata)
     {
       GeglTileHandlerCache *cache = userdata;
       cache->free_list = g_slist_prepend (cache->free_list, item);
-      
     }
 }
 
 static void
-dispose (GObject *object)
+gegl_tile_handler_cache_dispose (GObject *object)
 {
   GeglTileHandlerCache *cache;
   CacheItem            *item;
   GSList               *iter;
 
+  g_static_mutex_lock (&mutex);
   cache = GEGL_TILE_HANDLER_CACHE (object);
 
   /* only throw out items belonging to this cache instance */
 
   cache->free_list = NULL;
+  /* XXX: for optimization this could be delayed,. or collected among multiple
+   * buffer destructions, to avoid the overhead of walking the full queue for
+   * every tiny buffer being destroyed.
+   */
   g_queue_foreach (cache_queue, gegl_tile_handler_cache_dispose_buffer_tiles, cache);
   for (iter = cache->free_list; iter; iter = g_slist_next (iter))
     {
@@ -134,34 +148,40 @@ dispose (GObject *object)
         if (item->tile)
           {
             cache_total -= item->tile->size;
-            g_object_unref (item->tile);
+            gegl_tile_unref (item->tile);
           }
         g_queue_remove (cache_queue, item);
+        g_hash_table_remove (cache_ht, item);
         g_slice_free (CacheItem, item);
     }
   g_slist_free (cache->free_list);
   cache->free_list = NULL;
+  g_static_mutex_unlock (&mutex);
 
   G_OBJECT_CLASS (gegl_tile_handler_cache_parent_class)->dispose (object);
 }
 
 static GeglTile *
-get_tile (GeglTileSource *tile_store,
-          gint        x,
-          gint        y,
-          gint        z)
+gegl_tile_handler_cache_get_tile_command (GeglTileSource *tile_store,
+                                          gint        x,
+                                          gint        y,
+                                          gint        z)
 {
   GeglTileHandlerCache *cache    = GEGL_TILE_HANDLER_CACHE (tile_store);
-  GeglTileSource       *source = GEGL_HANDLER (tile_store)->source;
+  GeglTileSource       *source = GEGL_TILE_HANDLER (tile_store)->source;
   GeglTile             *tile     = NULL;
 
   tile = gegl_tile_handler_cache_get_tile (cache, x, y, z);
   if (tile)
     {
+#ifdef GEGL_DEBUG_CACHE_HITS
       cache_hits++;
+#endif
       return tile;
     }
+#ifdef GEGL_DEBUG_CACHE_HITS
   cache_misses++;
+#endif
 
   if (source)
     tile = gegl_tile_source_get_tile (source, x, y, z);
@@ -180,7 +200,7 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
                                  gint             z,
                                  gpointer         data)
 {
-  GeglTileHandler      *handler = GEGL_HANDLER (tile_store);
+  GeglTileHandler      *handler = GEGL_TILE_HANDLER (tile_store);
   GeglTileHandlerCache *cache   = GEGL_TILE_HANDLER_CACHE (handler);
 
   switch (command)
@@ -207,9 +227,9 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
          * generator after the cache, this would have to be possible to disable
          * to work in sync operation with backend.
          */
-        return get_tile (tile_store, x, y, z);
+        return gegl_tile_handler_cache_get_tile_command (tile_store, x, y, z);
       case GEGL_TILE_IS_CACHED:
-        return (gpointer)gegl_tile_handler_cache_has_tile (cache, x, y, z);
+        return GINT_TO_POINTER(gegl_tile_handler_cache_has_tile (cache, x, y, z));
       case GEGL_TILE_EXIST:
         {
           gboolean exist = gegl_tile_handler_cache_has_tile (cache, x, y, z);
@@ -221,7 +241,8 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
         {
           gboolean action = gegl_tile_handler_cache_wash (cache);
           if (action)
-            return (gpointer)action;
+            return GINT_TO_POINTER(action);
+          /* with no action, we chain up to lower levels */
           break;
         }
       case GEGL_TILE_REFETCH:
@@ -234,24 +255,7 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
         break;
     }
 
-  return gegl_tile_handler_chain_up (handler, command, x, y, z, data);
-}
-
-static void
-gegl_tile_handler_cache_class_init (GeglTileHandlerCacheClass *class)
-{
-  GObjectClass        *gobject_class = G_OBJECT_CLASS (class);
-  GeglTileSourceClass *source_class  = GEGL_TILE_SOURCE_CLASS (class);
-
-  gobject_class->finalize = finalize;
-  gobject_class->dispose  = dispose;
-  source_class->command   = gegl_tile_handler_cache_command;
-}
-
-static void
-gegl_tile_handler_cache_init (GeglTileHandlerCache *cache)
-{
-  gegl_tile_cache_init ();
+  return gegl_tile_handler_source_command (handler, command, x, y, z, data);
 }
 
 /* write the least recently used dirty tile to disk if it
@@ -295,30 +299,24 @@ gegl_tile_handler_cache_get_tile (GeglTileHandlerCache *cache,
                                   gint                  y,
                                   gint                  z)
 {
-  GList *link;
+  CacheItem *result;
+  CacheItem pin;
 
-  for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
+  pin.x = x;
+  pin.y = y;
+  pin.z = z;
+  pin.handler = cache;
+
+  g_static_mutex_lock (&mutex);
+  result = g_hash_table_lookup (cache_ht, &pin);
+  if (result)
     {
-      CacheItem *item = link->data;
-      GeglTile  *tile = item->tile;
-
-      if (tile != NULL &&
-          item->handler == cache &&
-          item->x == x &&
-          item->y == y &&
-          item->z == z)
-        {
-          /* move the link to the front of the queue */
-          if (link->prev != NULL)
-            {
-              g_queue_unlink (cache_queue, link);
-              g_queue_push_head_link (cache_queue, link);
-            }
-
-          return g_object_ref (tile);
-        }
+      g_queue_remove (cache_queue, result);
+      g_queue_push_head (cache_queue, result);
+      g_static_mutex_unlock (&mutex);
+      return gegl_tile_ref (result->tile);
     }
-
+  g_static_mutex_unlock (&mutex);
   return NULL;
 }
 
@@ -332,7 +330,7 @@ gegl_tile_handler_cache_has_tile (GeglTileHandlerCache *cache,
 
   if (tile)
     {
-      g_object_unref (G_OBJECT (tile));
+      gegl_tile_unref (tile);
       return TRUE;
     }
 
@@ -342,12 +340,15 @@ gegl_tile_handler_cache_has_tile (GeglTileHandlerCache *cache,
 static gboolean
 gegl_tile_handler_cache_trim (GeglTileHandlerCache *cache)
 {
-  CacheItem *last_writable = g_queue_pop_tail (cache_queue);
+  CacheItem *last_writable;
+ 
+  last_writable = g_queue_pop_tail (cache_queue);
 
   if (last_writable != NULL)
     {
+      g_hash_table_remove (cache_ht, last_writable);
       cache_total  -= last_writable->tile->size;
-      g_object_unref (last_writable->tile);
+      gegl_tile_unref (last_writable->tile);
       g_slice_free (CacheItem, last_writable);
       return TRUE;
     }
@@ -363,6 +364,7 @@ gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
 {
   GList *link;
 
+  g_static_mutex_lock (&mutex);
   for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
     {
       CacheItem *item = link->data;
@@ -376,13 +378,16 @@ gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
         {
           cache_total  -= item->tile->size;
           tile->tile_storage = NULL;
-          tile->stored_rev = tile->rev; /* to cheat it out of being stored */
-          g_object_unref (tile);
+          gegl_tile_mark_as_stored (tile); /* to cheat it out of being stored */
+          gegl_tile_unref (tile);
+          g_hash_table_remove (cache_ht, item);
           g_slice_free (CacheItem, item);
           g_queue_delete_link (cache_queue, link);
+          g_static_mutex_unlock (&mutex);
           return;
         }
     }
+  g_static_mutex_unlock (&mutex);
 }
 
 
@@ -397,6 +402,7 @@ gegl_tile_handler_cache_void (GeglTileHandlerCache *cache,
   if (!cache_queue)
     return;
 
+  g_static_mutex_lock (&mutex);
   for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
     {
       CacheItem *item = link->data;
@@ -409,13 +415,16 @@ gegl_tile_handler_cache_void (GeglTileHandlerCache *cache,
           item->handler == cache)
         {
           gegl_tile_void (tile);
-          cache_total  -= item->tile->size;
-          g_object_unref (tile);
+          cache_total -= item->tile->size;
+          gegl_tile_unref (tile);
+          g_hash_table_remove (cache_ht, item);
           g_slice_free (CacheItem, item);
           g_queue_delete_link (cache_queue, link);
+          g_static_mutex_unlock (&mutex);
           return;
         }
     }
+  g_static_mutex_unlock (&mutex);
 }
 
 void
@@ -426,23 +435,94 @@ gegl_tile_handler_cache_insert (GeglTileHandlerCache *cache,
                                 gint                  z)
 {
   CacheItem *item = g_slice_new (CacheItem);
-  guint      count;
 
   item->handler = cache;
-  item->tile    = g_object_ref (tile);
+  item->tile    = gegl_tile_ref (tile);
   item->x       = x;
   item->y       = y;
   item->z       = z;
-  cache_total  += item->tile->size;
 
+  g_static_mutex_lock (&mutex);
+  cache_total  += item->tile->size;
   g_queue_push_head (cache_queue, item);
 
-  count = g_queue_get_length (cache_queue);
+  g_hash_table_insert (cache_ht, item, item);
 
   while (cache_total > gegl_config()->cache_size)
     {
-      /*GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:%i > cache_size:%i", cache_total, gegl_config()->cache_size);
-      GEGL_NOTE(GEGL_DEBUG_CACHE, "%f%% hit:%i miss:%i  %i]", cache_hits*100.0/(cache_hits+cache_misses), cache_hits, cache_misses, g_queue_get_length (cache_queue));*/
+#ifdef GEGL_DEBUG_CACHE_HITS
+      GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:%i > cache_size:%i", cache_total, gegl_config()->cache_size);
+      GEGL_NOTE(GEGL_DEBUG_CACHE, "%f%% hit:%i miss:%i  %i]", cache_hits*100.0/(cache_hits+cache_misses), cache_hits, cache_misses, g_queue_get_length (cache_queue));
+#endif
       gegl_tile_handler_cache_trim (cache);
     }
+  g_static_mutex_unlock (&mutex);
+}
+
+GeglTileHandlerCache *
+gegl_tile_handler_cache_new (void)
+{
+  return g_object_new (GEGL_TYPE_TILE_HANDLER_CACHE, NULL);
+}
+
+
+static guint
+gegl_tile_handler_cache_hashfunc (gconstpointer key)
+{
+  const CacheItem *e = key;
+  guint           hash;
+  gint            i;
+  gint            srcA = e->x;
+  gint            srcB = e->y;
+  gint            srcC = e->z;
+
+  /* interleave the 10 least significant bits of all coordinates,
+   * this gives us Z-order / morton order of the space and should
+   * work well as a hash
+   */
+  hash = 0;
+  for (i = 9; i >= 0; i--)
+    {
+#define ADD_BIT(bit)    do { hash |= (((bit) != 0) ? 1 : 0); hash <<= 1; } while (0)
+      ADD_BIT (srcA & (1 << i));
+      ADD_BIT (srcB & (1 << i));
+      ADD_BIT (srcC & (1 << i));
+#undef ADD_BIT
+    }
+  return hash ^ GPOINTER_TO_INT (e->handler);
+}
+
+static gboolean
+gegl_tile_handler_cache_equalfunc (gconstpointer a,
+                                   gconstpointer b)
+{
+  const CacheItem *ea = a;
+  const CacheItem *eb = b;
+
+  if (ea->x == eb->x &&
+      ea->y == eb->y &&
+      ea->z == eb->z &&
+      ea->handler == eb->handler)
+    return TRUE;
+  return FALSE;
+}
+
+void
+gegl_tile_cache_init (void)
+{
+  if (cache_queue == NULL)
+    cache_queue = g_queue_new ();
+  if (cache_ht == NULL)
+    cache_ht = g_hash_table_new (gegl_tile_handler_cache_hashfunc, gegl_tile_handler_cache_equalfunc);
+}
+
+void
+gegl_tile_cache_destroy (void)
+{
+  if (cache_queue)
+    g_queue_free (cache_queue);
+  if (cache_ht)
+    g_hash_table_destroy (cache_ht);
+  cache_queue = NULL;
+  cache_ht = NULL;
 }
