@@ -20,6 +20,8 @@
 
 #include <glib.h>
 
+#include "glib-object.h"
+#include "gegl-buffer-types.h"
 #include "gegl-tile-handler-chain.h"
 #include "gegl-tile-handler-cache.h"
 
