@@ -37,17 +37,15 @@ enum
   PROP_SWAP,
   PROP_BABL_TOLERANCE,
   PROP_TILE_WIDTH,
-  PROP_TILE_HEIGHT
-#if ENABLE_MT
-  ,PROP_THREADS
-#endif
+  PROP_TILE_HEIGHT,
+  PROP_THREADS
 };
 
 static void
-get_property (GObject    *gobject,
-              guint       property_id,
-              GValue     *value,
-              GParamSpec *pspec)
+gegl_config_get_property (GObject    *gobject,
+                          guint       property_id,
+                          GValue     *value,
+                          GParamSpec *pspec)
 {
   GeglConfig *config = GEGL_CONFIG (gobject);
 
@@ -81,11 +79,9 @@ get_property (GObject    *gobject,
         g_value_set_string (value, config->swap);
         break;
 
-#if ENABLE_MT
       case PROP_THREADS:
         g_value_set_int (value, config->threads);
         break;
-#endif
 
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -94,10 +90,10 @@ get_property (GObject    *gobject,
 }
 
 static void
-set_property (GObject      *gobject,
-              guint         property_id,
-              const GValue *value,
-              GParamSpec   *pspec)
+gegl_config_set_property (GObject      *gobject,
+                          guint         property_id,
+                          const GValue *value,
+                          GParamSpec   *pspec)
 {
   GeglConfig *config = GEGL_CONFIG (gobject);
 
@@ -135,11 +131,9 @@ set_property (GObject      *gobject,
          g_free (config->swap);
         config->swap = g_value_dup_string (value);
         break;
-#if ENABLE_MT
       case PROP_THREADS:
         config->threads = g_value_get_int (value);
         return;
-#endif
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -147,7 +141,7 @@ set_property (GObject      *gobject,
 }
 
 static void
-finalize (GObject *gobject)
+gegl_config_finalize (GObject *gobject)
 {
   GeglConfig *config = GEGL_CONFIG (gobject);
 
@@ -164,9 +158,9 @@ gegl_config_class_init (GeglConfigClass *klass)
 
   parent_class  = g_type_class_peek_parent (klass);
 
-  gobject_class->set_property = set_property;
-  gobject_class->get_property = get_property;
-  gobject_class->finalize = finalize;
+  gobject_class->set_property = gegl_config_set_property;
+  gobject_class->get_property = gegl_config_get_property;
+  gobject_class->finalize     = gegl_config_finalize;
 
 
   g_object_class_install_property (gobject_class, PROP_TILE_WIDTH,
@@ -205,12 +199,10 @@ gegl_config_class_init (GeglConfigClass *klass)
                                    g_param_spec_string ("swap", "Swap", "where gegl stores it's swap files", NULL,
                                                      G_PARAM_READWRITE));
 
-#if ENABLE_MT
-  g_object_class_install_property (gobject_class, PROP_TILE_HEIGHT,
+  g_object_class_install_property (gobject_class, PROP_THREADS,
                                    g_param_spec_int ("threads", "Number of concurrent evaluation threads", "default tile height for created buffers.",
-                                                     0, 16, 2,
+                                                     0, 16, 1,
                                                      G_PARAM_READWRITE));
-#endif
 }
 
 static void
@@ -222,7 +214,5 @@ gegl_config_init (GeglConfig *self)
   self->chunk_size  = 512 * 512;
   self->tile_width  = 128;
   self->tile_height = 64;
-#if ENABLE_MT
-  self->threads = 2;
-#endif
+  self->threads = 1;
 }
