@@ -127,6 +127,8 @@
   gegl_rectangle_contains (const GeglRectangle *r,
                            const GeglRectangle *s)
   {
+    g_return_val_if_fail (r && s, FALSE);
+
     if (s->x >= r->x &&
         s->y >= r->y &&
         (s->x + s->width) <= (r->x + r->width) &&
@@ -140,6 +142,8 @@
   gegl_rectangle_equal (const GeglRectangle *r,
                         const GeglRectangle *s)
   {
+    g_return_val_if_fail (r && s, FALSE);
+
     if (r->x == s->x &&
         r->y == s->y &&
         r->width == s->width &&
@@ -156,6 +160,8 @@
                                gint                 w,
                                gint                 h)
   {
+    g_return_val_if_fail (r, FALSE);
+
     if (r->x == x &&
         r->y == y &&
         r->width == w &&
@@ -165,6 +171,13 @@
       return FALSE;
   }
 
+  gboolean
+  gegl_rectangle_is_empty (const GeglRectangle *r)
+  {
+    g_return_val_if_fail (r != NULL, FALSE);
+    return r->width == 0 && r->height == 0;
+  }
+
   static GeglRectangle *
   gegl_rectangle_dup (const GeglRectangle *rectangle)
   {
@@ -228,7 +241,7 @@
     gint   offset;
 
     mem    = g_malloc (size + GEGL_ALIGN + sizeof(gpointer));
-    offset = GEGL_ALIGN - (((guint)mem) + sizeof(gpointer)) % GEGL_ALIGN;
+    offset = GEGL_ALIGN - (GPOINTER_TO_UINT(mem) + sizeof(gpointer)) % GEGL_ALIGN;
     ret    = (gpointer)(mem + sizeof(gpointer) + offset);
 
     /* store the real malloc one pointer in front of this malloc */
