@@ -17,6 +17,9 @@
  */
 #include "config.h"
 
+/* For clang, remove when getting rid of using PATH_MAX */
+#define _XOPEN_SOURCE 500
+
 #include <glib.h>
 #include <glib-object.h>
 #include <stdlib.h>
@@ -52,6 +55,7 @@ enum
   STATE_TREE_FIRST_CHILD
 };
 
+
 struct _ParseData
 {
   gint         state;
@@ -68,6 +72,10 @@ struct _ParseData
   GList       *refs;
 };
 
+
+/* Search a paired attribute name/value arrays for an entry with a specific
+ * name, return the value or null if not found.
+ */
 static const gchar *name2val (const gchar **attribute_names,
                               const gchar **attribute_values,
                               const gchar  *name)
@@ -127,20 +135,30 @@ set_clone_prop_as_well:
             {
               gegl_node_set (new, param_name, param_value, NULL);
             }
-          else if (pd->path_root)
-            {
-              gchar absolute_path[PATH_MAX];
-              g_snprintf (buf, sizeof (buf),
-                          "%s/%s", pd->path_root, param_value);
-              realpath (buf, absolute_path);
-              gegl_node_set (new, param_name, absolute_path, NULL);
-            }
           else
             {
-              gchar absolute_path[PATH_MAX];
-              g_snprintf (buf, sizeof (buf), "./%s", param_value);
-              realpath (buf, absolute_path);
-              gegl_node_set (new, param_name, absolute_path, NULL);
+              gchar * absolute_path;
+              if (pd->path_root)
+                {
+                  g_snprintf (buf, sizeof (buf),
+                              "%s/%s", pd->path_root, param_value);
+                }
+              else
+                {
+                  g_snprintf (buf, sizeof (buf), "./%s", param_value);
+                }
+
+              absolute_path = realpath (buf, NULL);
+              if (absolute_path)
+                {
+                  gegl_node_set (new, param_name, absolute_path, NULL);
+                  free (absolute_path);
+                }
+              else
+                {
+                  g_warning ("Unable to obtain absolute path for paramater %s\n",
+                             param_name);
+                }
             }
         }
       else if (paramspec->value_type == G_TYPE_INT)
@@ -209,6 +227,30 @@ set_clone_prop_as_well:
     }
 }
 
+
+/* Check that the attributes for ELEMENT include an attribute of name NAME,
+ * and store it in the variable of the same name. Else report an error.
+ *
+ * eg:  { collect_attribute (value, 'key'); }
+ */
+#define collect_attribute(NAME, ELEMENT)                                \
+{                                                                       \
+  const gchar *__value = name2val (a, v, (G_STRINGIFY (NAME)));         \
+  if (!__value)                                                         \
+    {                                                                   \
+      *error = g_error_new (G_MARKUP_ERROR,                             \
+                            G_MARKUP_ERROR_MISSING_ATTRIBUTE,           \
+                            "Expected attribute '%s' in element '%s'",  \
+                            (G_STRINGIFY (NAME)),                       \
+                            (ELEMENT));                                 \
+      return;                                                           \
+    }                                                                   \
+  else                                                                  \
+    {                                                                   \
+      (NAME) = __value;                                                 \
+    }                                                                   \
+}
+
 static void start_element (GMarkupParseContext *context,
                            const gchar         *element_name,
                            const gchar        **attribute_names,
@@ -249,19 +291,24 @@ static void start_element (GMarkupParseContext *context,
     }
   else if (!strcmp (element_name, "param"))
     {
+      const gchar *name;
       if (pd->param != NULL)
         g_warning ("eek, haven't cleared previous param");
-      g_assert (name2val (a, v, "name"));
-      pd->param = g_strdup (name2val (a, v, "name"));
+
+      collect_attribute (name, "param");
+      pd->param = g_strdup (name);
     }
   else if (!strcmp (element_name, "curve"))
     {
+      const gchar *ymin, *ymax;
       if (pd->curve != NULL)
 	g_warning ("we haven't cleared previous curve");
-      g_assert (name2val (a, v, "ymin"));
-      g_assert (name2val (a, v, "ymax"));
-      pd->curve = gegl_curve_new (g_ascii_strtod (name2val (a, v, "ymin"), NULL),
-				  g_ascii_strtod (name2val (a, v, "ymax"), NULL));
+
+      collect_attribute (ymin, "curve");
+      collect_attribute (ymax, "curve");
+
+      pd->curve = gegl_curve_new (g_ascii_strtod (ymin, NULL),
+				  g_ascii_strtod (ymax, NULL));
     }
   else if (!strcmp (element_name, "curve-point"))
     {
@@ -269,12 +316,13 @@ static void start_element (GMarkupParseContext *context,
 	g_warning ("curve not instantiated");
       else
         {
-	  g_assert (name2val (a, v, "x"));
-	  g_assert (name2val (a, v, "y"));
+          const gchar *x, *y;
+          collect_attribute (x, "curve-point");
+          collect_attribute (y, "curve-point");
 
 	  gegl_curve_add_point (pd->curve,
-				g_ascii_strtod (name2val (a, v, "x"), NULL),
-				g_ascii_strtod (name2val (a, v, "y"), NULL));
+				g_ascii_strtod (x, NULL),
+				g_ascii_strtod (y, NULL));
 	}
     }
   else if (!strcmp (element_name, "link") ||
@@ -305,15 +353,15 @@ static void start_element (GMarkupParseContext *context,
         }
       else if (!strcmp (element_name, "node"))
         {
-          new = gegl_node_new_child (pd->gegl,
-                                     "operation", name2val (a, v, "operation"),
-                                     NULL);
+          const gchar *operation;
+          collect_attribute (operation, "node");
+          new = gegl_node_new_child (pd->gegl, "operation", operation, NULL);
         }
       else if (!strcmp (element_name, "filter"))
         {
-          new = gegl_node_new_child (pd->gegl,
-                                     "operation", name2val (a, v, "type"),
-                                     NULL);
+          const gchar *type;
+          collect_attribute (type, "filter");
+          new = gegl_node_new_child (pd->gegl, "operation", type, NULL);
         }
       else
         {
@@ -321,7 +369,16 @@ static void start_element (GMarkupParseContext *context,
                                      "operation", element_name,
                                      NULL);
         }
-      g_assert (new);
+
+      /* Could not instance an appropriate node */
+      if (!new)
+        {
+          *error = g_error_new (G_MARKUP_ERROR,
+                                G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                                "Could not instantiate operation '%s'",
+                                element_name);
+          return;
+        }
 
       while (*a)
         {
@@ -469,55 +526,74 @@ GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
   glong                time = gegl_ticks ();
   ParseData            pd   = { 0, };
   GMarkupParseContext *context;
+  gboolean             success = FALSE;
+
+  g_return_val_if_fail (xmldata != NULL, NULL);
 
   pd.ids       = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   pd.refs      = NULL;
   pd.path_root = path_root;
 
-  context = g_markup_parse_context_new (&parser, 0, &pd, NULL);
-  g_markup_parse_context_parse (context, xmldata, strlen (xmldata), NULL);
-
-  /* connect clones */
-  g_list_foreach (pd.refs, each_ref, &pd);
+  g_list_free (pd.refs);
+  context = g_markup_parse_context_new   (&parser, 0, &pd, NULL);
+  success = g_markup_parse_context_parse (context,
+                                          xmldata,
+                                          strlen (xmldata),
+                                          NULL);
+  if (success)
+    {
+      /* connect clones */
+      g_list_foreach (pd.refs, each_ref, &pd);
+    }
+  else
+    {
+      if (pd.gegl)
+        {
+          g_object_unref (pd.gegl);
+          pd.gegl = NULL;
+        }
+    }
 
+  g_list_free (pd.refs);
+  g_list_free (pd.parent);
   g_markup_parse_context_free (context);
   g_hash_table_destroy (pd.ids);
 
   time = gegl_ticks () - time;
   gegl_instrument ("gegl", "gegl_parse_xml", time);
 
-  return GEGL_NODE (pd.gegl);
+  return success ? GEGL_NODE (pd.gegl) : NULL;
 }
 
 GeglNode *
 gegl_node_new_from_file (const gchar   *path)
 {
-  GeglNode *node;
-  GError   *err = NULL;
+  GeglNode *node = NULL;
+  GError   *err  = NULL;
   gchar    *script;
-  gchar    *temp1;
-  gchar    *temp2;
-  gchar    *path_root;
+  gchar     path_root[PATH_MAX];
+  gchar    *dirname;
 
   g_assert (path);
 
-  temp1 = g_strdup (path);
-  temp2 = g_strdup (g_path_get_dirname (temp1));
-  path_root = g_strdup (realpath (temp2, NULL));
+  dirname = g_path_get_dirname (path);
+  if (!realpath (dirname, path_root))
+    {
+      goto cleanup;
+    }
 
   g_file_get_contents (path, &script, NULL, &err);
   if (err != NULL)
     {
       g_warning ("Unable to read file: %s", err->message);
       g_error_free (err);
-      return NULL;
+      goto cleanup;
     }
 
   node = gegl_node_new_from_xml (script, path_root);
 
-  g_free (temp1);
-  g_free (temp2);
-  g_free (path_root);
+cleanup:
+  g_free (dirname);
   return node;
 }
 
@@ -778,9 +854,7 @@ serialize_properties (SerializeState *ss,
 	  else if (properties[i]->value_type == GEGL_TYPE_CURVE)
 	    {
 	      GeglCurve *curve;
-	      guint num_points;
 	      gegl_node_get (node, properties[i]->name, &curve, NULL);
-	      num_points = gegl_curve_num_points (curve);
 	      xml_param_start (ss, indent + 2, properties[i]->name);
 	      g_string_append (ss->buf, "\n");
 	      xml_curve (ss, indent + 4, curve);
