@@ -75,6 +75,8 @@ struct _GeglNode
   /* Whether result is cached or not, inherited by children */
   gboolean        dont_cache;
 
+  GMutex         *mutex;
+
   /*< private >*/
   GeglNodePrivate *priv;
 };
@@ -141,6 +143,8 @@ void          gegl_node_add_pad             (GeglNode      *self,
                                              GeglPad       *pad);
 void          gegl_node_remove_pad          (GeglNode      *self,
                                              GeglPad       *pad);
+gboolean      gegl_node_has_pad             (GeglNode      *self,
+                                             const gchar   *name);
 GeglPad     * gegl_node_get_pad             (GeglNode      *self,
                                              const gchar   *name);
 GSList      * gegl_node_get_pads            (GeglNode      *self);
@@ -150,7 +154,6 @@ gint          gegl_node_get_num_sinks       (GeglNode      *self);
 GeglNode    * gegl_node_get_producer        (GeglNode      *self,
                                              gchar         *pad_name,
                                              gchar        **output_pad);
-GSList      * gegl_node_get_depends_on      (GeglNode      *self);
 void          gegl_node_dump_depends_on     (GeglNode      *self);
 void          gegl_node_set_valist          (GeglNode      *object,
                                              const gchar   *first_property_name,
@@ -219,6 +222,14 @@ const gchar * gegl_node_get_name            (GeglNode      *self);
 void          gegl_node_set_name            (GeglNode      *self,
                                              const gchar   *name);
 
+/* macros used to set flags on objects passed in the graph */
+
+#define gegl_object_set_has_forked(object) \
+      g_object_set_data(G_OBJECT(object), "gegl has-forked", (void*)0xf)
+#define gegl_object_get_has_forked(object) \
+      (g_object_get_data(G_OBJECT(object), "gegl has-forked")!=NULL)
+
+#define GEGL_MAX_THREADS 16
 
 G_END_DECLS
 
