@@ -16,20 +16,17 @@
  * gimpmodule.c: * (C) 1999 Austin Donnelly <austin@gegl.org>
  */
 
+#include "config.h"
 
 #include <string.h>
 
 #include <glib-object.h>
+#include <glib/gi18n-lib.h>
 
 #include "geglmodule.h"
 
-/* define hacks to make it compile with as small modifications as possible
- * from geglmodule
- */
-#define _(string)                       (string)
-#define N_(string)                      (string)
+
 #define gegl_filename_to_utf8(filename) (filename)
-#define gettext(string)                 (string)
 
 enum
 {
@@ -298,7 +295,7 @@ gegl_module_query_module (GeglModule *module)
     {
       gegl_module_set_last_error (module,
                                   info ?
-                                  "module ABI version does not match" :
+                                  "module ABI version does not match op not loaded, to get rid of this warning remove (clean/uninstall) .so files in GEGLs search path." :
                                   "gegl_module_query() returned NULL");
 
       g_message (_("Module '%s' load error: %s"),
@@ -443,23 +440,11 @@ gegl_module_set_last_error (GeglModule  *module,
  * Return value: The new #GeglModuleInfo struct.
  **/
 GeglModuleInfo *
-gegl_module_info_new (guint32      abi_version,
-                      const gchar *purpose,
-                      const gchar *author,
-                      const gchar *version,
-                      const gchar *copyright,
-                      const gchar *date)
+gegl_module_info_new (guint32 abi_version)
 {
-  GeglModuleInfo *info;
-
-  info = g_new0 (GeglModuleInfo, 1);
+  GeglModuleInfo *info = g_slice_new0 (GeglModuleInfo);
 
   info->abi_version = abi_version;
-  info->purpose     = g_strdup (purpose);
-  info->author      = g_strdup (author);
-  info->version     = g_strdup (version);
-  info->copyright   = g_strdup (copyright);
-  info->date        = g_strdup (date);
 
   return info;
 }
@@ -477,12 +462,7 @@ gegl_module_info_copy (const GeglModuleInfo *info)
 {
   g_return_val_if_fail (info != NULL, NULL);
 
-  return gegl_module_info_new (info->abi_version,
-                               info->purpose,
-                               info->author,
-                               info->version,
-                               info->copyright,
-                               info->date);
+  return gegl_module_info_new (info->abi_version);
 }
 
 /**
@@ -496,13 +476,7 @@ gegl_module_info_free (GeglModuleInfo *info)
 {
   g_return_if_fail (info != NULL);
 
-  g_free (info->purpose);
-  g_free (info->author);
-  g_free (info->version);
-  g_free (info->copyright);
-  g_free (info->date);
-
-  g_free (info);
+  g_slice_free (GeglModuleInfo, info);
 }
 
 GType
