@@ -76,8 +76,6 @@ struct _GeglProcessor
   GSList          *dirty_rectangles;
   gint             chunk_size;
 
-  GThread         *thread;
-  gboolean         thread_done;
   gdouble          progress;
 };
 
@@ -160,6 +158,12 @@ gegl_processor_finalize (GObject *self_object)
 {
   GeglProcessor *processor = GEGL_PROCESSOR (self_object);
 
+  if (processor->context)
+    {
+      GeglCache *cache = gegl_node_get_cache (processor->input);
+      gegl_node_remove_context (processor->node, cache);
+    }
+
   if (processor->node)
     {
       g_object_unref (processor->node);
@@ -194,7 +198,7 @@ gegl_processor_set_property (GObject      *gobject,
   switch (property_id)
     {
       case PROP_NODE:
-        gegl_processor_set_node (self, g_value_dup_object (value));
+        gegl_processor_set_node (self, g_value_get_object (value));
         break;
 
       case PROP_CHUNK_SIZE:
@@ -278,6 +282,8 @@ gegl_processor_set_node (GeglProcessor *processor,
     }
 
   g_object_ref (processor->input);
+
+  g_object_notify (G_OBJECT (processor), "node");
 }
 
 
@@ -301,7 +307,7 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
       rectangle          = &input_bounding_box;
     }
 
-  GEGL_NOTE (GEGL_DEBUG_PROCESS, "gegl_processor_set_rectangle() node = %s rectangle = %d, %d %d×%d\n",
+  GEGL_NOTE (GEGL_DEBUG_PROCESS, "gegl_processor_set_rectangle() node = %s rectangle = %d, %d %d×%d",
              gegl_node_get_debug_name (processor->node),
              rectangle->x, rectangle->y, rectangle->width, rectangle->height);
 
@@ -332,7 +338,8 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
   /* if the node's operation is a sink and it needs the full content then
    * a context will be set up together with a cache and
    * needed and result rectangles */
-  if (GEGL_IS_OPERATION_SINK (processor->node->operation) &&
+  if (processor->node &&
+      GEGL_IS_OPERATION_SINK (processor->node->operation) &&
       gegl_operation_sink_needs_full (processor->node->operation))
     {
       GeglCache *cache;
@@ -354,6 +361,8 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
       gegl_operation_context_set_need_rect   (processor->context,
                                               &processor->rectangle);
     }
+
+  g_object_notify (G_OBJECT (processor), "rectangle");
 }
 
 /* Will generate band_sizes that are adapted to the size of the tiles */
@@ -480,6 +489,7 @@ render_rectangle (GeglProcessor *processor)
               /* release the buffer */
               g_free (buf);
             }
+          g_slice_free (GeglRectangle, dr);
         }
       else
         {
@@ -598,6 +608,7 @@ gegl_processor_render (GeglProcessor *processor,
     }
   else
     {
+      g_return_val_if_fail (processor->input != NULL, FALSE);
       valid_region = gegl_node_get_cache (processor->input)->valid_region;
     }
 
@@ -628,7 +639,6 @@ gegl_processor_render (GeglProcessor *processor,
               {
                 *progress = (double) valid / wanted;
               }
-            wanted = 1;
           }
 
         return more_work;
