@@ -34,6 +34,43 @@ gegl_chant_file_path (path, _("File"), "", _("Path of file to load."))
 #include "gegl-chant.h"
 #include <png.h>
 
+static FILE * open_png(const gchar *path)
+{
+  FILE *infile;
+  const size_t hdr_size=8;
+  size_t hdr_read_size;
+  unsigned char header[hdr_size];
+
+  if (!strcmp (path, "-"))
+    {
+      infile = stdin;
+    }
+  else
+    {
+      infile = fopen (path, "rb");
+    }
+  if (!infile)
+    {
+      return infile;
+    }
+
+  if((hdr_read_size=fread(header, 1, hdr_size, infile))!=hdr_size)
+    {
+      fclose(infile);
+      g_warning ("%s is too short for a png file, only %lu bytes.",
+                 path, (unsigned long)hdr_read_size);
+      return NULL;
+    }
+
+  if (png_sig_cmp (header, 0, hdr_size))
+    {
+      fclose (infile);
+      g_warning ("%s is not a png file", path);
+      return NULL;
+    }
+  return infile;
+}
+
 static gint
 gegl_buffer_import_png (GeglBuffer  *gegl_buffer,
                         const gchar *path,
@@ -53,7 +90,6 @@ gegl_buffer_import_png (GeglBuffer  *gegl_buffer,
   FILE          *infile;
   png_structp    load_png_ptr;
   png_infop      load_info_ptr;
-  unsigned char  header[8];
   guchar        *pixels;
   /*png_bytep     *rows;*/
 
@@ -61,27 +97,13 @@ gegl_buffer_import_png (GeglBuffer  *gegl_buffer,
   unsigned   int i;
   png_bytep  *row_p = NULL;
 
-  if (!strcmp (path, "-"))
-    {
-      infile = stdin;
-    }
-  else
-    {
-      infile = fopen (path, "rb");
-    }
+  infile = open_png (path);
+
   if (!infile)
     {
       return -1;
     }
 
-  fread (header, 1, 8, infile);
-  if (png_sig_cmp (header, 0, 8))
-    {
-      fclose (infile);
-      g_warning ("%s is not a png file", path);
-      return -1;
-    }
-
   load_png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 
   if (!load_png_ptr)
@@ -181,7 +203,7 @@ gegl_buffer_import_png (GeglBuffer  *gegl_buffer,
     if (interlace_type == PNG_INTERLACE_ADAM7)
       number_of_passes = png_set_interlace_handling (load_png_ptr);
 
-    if (load_info_ptr->valid & PNG_INFO_gAMA)
+    if (png_get_valid (load_png_ptr, load_info_ptr, PNG_INFO_gAMA))
       {
         gdouble gamma;
         png_get_gAMA (load_png_ptr, load_info_ptr, &gamma);
@@ -239,24 +261,16 @@ static gint query_png (const gchar *path,
   FILE         *infile;
   png_structp   load_png_ptr;
   png_infop     load_info_ptr;
-  unsigned char header[8];
 
   png_bytep  *row_p = NULL;
 
-  infile = fopen (path, "rb");
+  infile = open_png (path);
+
   if (!infile)
     {
       return -1;
     }
 
-  fread (header, 1, 8, infile);
-  if (png_sig_cmp (header, 0, 8))
-    {
-      fclose (infile);
-      g_warning ("%s is not a png file", path);
-      return -1;
-    }
-
   load_png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if (!load_png_ptr)
     {
@@ -298,7 +312,7 @@ static gint query_png (const gchar *path,
     *width = w;
     *height = h;
 
-    if (load_info_ptr->valid & PNG_INFO_tRNS)
+    if (png_get_valid (load_png_ptr, load_info_ptr, PNG_INFO_tRNS))
       color_type |= PNG_COLOR_MASK_ALPHA;
 
     if (color_type & PNG_COLOR_TYPE_RGB)
@@ -435,7 +449,6 @@ gegl_chant_class_init (GeglChantClass *klass)
     if (done)
       return; */
   gegl_extension_handler_register (".png", "gegl:png-load");
-  gegl_extension_handler_register (".PNG", "gegl:png-load");
 /*  done = TRUE; */
 }
 
