@@ -188,7 +188,7 @@ process (GeglOperation       *operation,
   cairo_translate (cr, -result->x, -result->y);
   text_layout_text (self, cr, 0, NULL, NULL);
 
-  gegl_buffer_set (output, NULL, babl_format ("B'aG'aR'aA u8"), data,
+  gegl_buffer_set (output, result, babl_format ("B'aG'aR'aA u8"), data,
                    GEGL_AUTO_ROWSTRIDE);
 
   cairo_destroy (cr);
@@ -246,6 +246,8 @@ get_bounding_box (GeglOperation *operation)
       /* store the measured size for later use */
       o->width = width;
       o->height = height;
+
+      gegl_operation_invalidate (operation, NULL, TRUE);
     }
 
   if (status)
